<template>
  <div class="el-menu-horizontal-warp">
    <!-- 横向菜单 -->
    <el-scrollbar @wheel.native.prevent="onElMenuHorizontalScroll" ref="elMenuHorizontalScrollRef">
      <el-menu router :default-active="state.defaultActive" :ellipsis="false" background-color="transparent"
        mode="horizontal">
        <template v-for="val in menuLists">
          <!-- v-if显示有二级菜单 -->
          <el-sub-menu :index="val.path" v-if="val.children && val.children.length > 0" :key="val.path">
            <template #title>
              <SvgIcon :name="val.meta.icon"></SvgIcon>
              <span>{{ $t(val.meta.title) }}</span>
            </template>
            <SubItem :chil="val.children" />
          </el-sub-menu>
          <!-- v-else没有二级菜单 :index做点击跳转路由的-->
          <template v-else>
            <el-menu-item :index='val.path' :key="val.path">
              <!-- v-if 首页与嵌套iframe -->
              <template #title v-if="!val.meta.isLink || (val.meta.isLink && val.meta.isIframe)">
                <SvgIcon :name="val.meta.icon" />
                {{ $t(val.meta.title) }}
              </template>
              <!-- v-else 不是首页也不是嵌套iframe，也没有二级菜单，val.meta.isLink有第三方链接地址，菜单直接点击先跳转对应当前的路由，再val.meta.isLink跳转第三方地址,打开外部链接 -->
              <template #title v-else>
                <a class="w100" @click.prevent="onALinkClick(val)">
                  <SvgIcon :name="val.meta.icon" />
                  {{ $t(val.meta.title) }}
                </a>
              </template>
            </el-menu-item>
          </template>
        </template>
      </el-menu>
    </el-scrollbar>
  </div>
</template>

<script setup lang="ts" name="navMenuHorizontal">
import { storeToRefs } from 'pinia';
import { computed, defineAsyncComponent, ref, onBeforeMount, nextTick, reactive, onMounted } from 'vue'
import { RouteRecordRaw, useRoute, onBeforeRouteUpdate } from 'vue-router'
import { useThemeConfig } from '/@/stores/themeConfig'
import other from '/@/utils/other';
import mittBus from '/@/utils/mitt'

import { useRoutesList } from '/@/stores/routesList'

// 引入组件
const SubItem = defineAsyncComponent(() => import('/@/layout/navMenu/subItem.vue'));

// 定义变量内容
const elMenuHorizontalScrollRef = ref()
const state = reactive({
  defaultActive: '' as string | undefined,
})
const storesThemeConfig = useThemeConfig()
const { themeConfig } = storeToRefs(storesThemeConfig)
const route = useRoute()

//下面两行一体，获取store状态管理--路由列表
const stores = useRoutesList()
const { routesList } = storeToRefs(stores)

// 定义父组件传过来的值
const props = defineProps({
  //菜单列表
  menuList: {
    type: Array<RouteRecordRaw>,
    default: () => [],
  }
})
// 获取父级菜单数据
const menuLists = computed(() => {
  return <RouteItems>props.menuList;//global.d.ts声明RouteItems类型
})

/** 设置横向滚动条可以鼠标滚轮滚动
 * @scroll 的用法:在我们滚动鼠标滚轮或者以任何方式滚动窗口条的时候，该事件就会被触发
 * @wheel  :只有当我们滚动鼠标滚轮的时候，才会触发;而且，如果我们触发的事件是一个很复杂的事件，那么只有在这个事件执行完毕之后，滚动条才会向下移动！
 * 为了不这样，我们给 @wheel 设定了一个特殊的属性：passive，它能够让滚动先执行，再去执行滚动触发的事件，这样页面就不会发生卡顿 * 
 *<ul @wheel.passive="xxx"></ul>  事件的默认行为立即执行，无需等待事件回调执行完毕  
 */
const onElMenuHorizontalScroll = (e: WheelEventType) => {
  console.log(e)
  const eventDelta = e.wheelDelta || -e.deltaY * 40;
  //https://cn.vuejs.org/api/component-instance.html#refs   一个包含 DOM 元素和组件实例的 对象，通过模板引用注册。
  //$refs 可以把proxy代理转为object对象
  elMenuHorizontalScrollRef.value.$refs.wrapRef.scrollLeft = elMenuHorizontalScrollRef.value.$refs.wrapRef.scrollLeft + eventDelta / 4;
  console.log(elMenuHorizontalScrollRef.value.$refs) //$refs 可以把proxy代理转为object对象
  console.log(elMenuHorizontalScrollRef.value.$refs.wrapRef.scrollLeft)//或者写成$refs也可以用
}
// 打开外部链接
const onALinkClick = (val: RouteItem) => {
  other.handleOpenLink(val)
}

// 设置页面当前路由高亮
const setCurrentRouterHighlight = (currentRoute: RouteToFrom) => {
  const { path, meta } = currentRoute;
  if (themeConfig.value.layout === 'classic') {//如果是经典布局
    state.defaultActive = `/${path?.split('/')[1]}`// path:'/home'-->获取到home
  } else {
    const pathSplit = meta?.isDynamic ? meta.isDynamicPath!.split('/') : path!.split('/')
    if (pathSplit.length >= 4 && meta?.isHide) state.defaultActive = pathSplit.splice(0, 3).join('/');
    else state.defaultActive = path
  }
}
// 初始化数据，页面刷新时，滚动条滚动到对应位置
const initElMenuOffsetLeft = () => {
  nextTick(() => {
    let els = <HTMLElement>document.querySelector('.el-menu.el-menu--horizontal li.is-active');
    if (!els) return false;
    elMenuHorizontalScrollRef.value.wrapRef.scrollLeft = els.offsetLeft;//获取当前高亮li偏移左边距离
  })
}

// 传送当前子级数据到菜单中
const setSendClassicChildren = (path: string) => {
  debugger
  const currentPathSplit = path.split('/')
  let currentData: MittMenu = { children: [] }
  filterRoutesFun(routesList.value).map((v, k) => {
    if (v.path === `/${currentPathSplit[1]}`) {
      v['k'] = k //待研究55
      currentData['item'] = { ...v }
      currentData['children'] = [{ ...v }]
      if (v.children) currentData['children'] = v.children
    }
  });
  return currentData;
}

// 路由过滤递归函数--》Object.assign()方法用于对象的合并，将源对象（source）的所有可枚举属性，复制到目标对象（target）
const filterRoutesFun = <T extends RouteItem>(arr: T[]): T[] => {
  //最终返回数组
  return arr.filter((item: T) => !item.meta?.isHide).map((item: T) => {
    item = Object.assign({}, item);//Object.assign()拷贝的那个属性而言是深拷贝；如果对象的属性值为引用，对于通过通过Object.assign()拷贝的那个属性而言其实是浅拷贝的
    if (item.children) item.children = filterRoutesFun(item.children)
    return item;
  })
}

// 页面加载前-->一进入页面默认高亮所在的路由
onBeforeMount(() => {
  setCurrentRouterHighlight(route)
})
// 页面加载时-->初始化数据，页面刷新时，滚动条滚动到对应位置
onMounted(() => {
  initElMenuOffsetLeft()
})
// 路由更新时-->动态路由带参数，router-link跳转
onBeforeRouteUpdate((to, from) => {//待研究555
  // 修复：https://gitee.com/lyt-top/vue-next-admin/issues/I3YX6G 
  console.log(to)
  console.log(from)
  setCurrentRouterHighlight(to)
  // 修复经典布局开启切割菜单时，点击tagsView后左侧导航菜单数据不变的问题---》重置清空数组，再赋值数组，相当于刷新最新数据
  let { layout, isClassicSplitMenu } = themeConfig.value
  if (layout === 'classic' && isClassicSplitMenu) {
    mittBus.emit('setSendClassicChildren', setSendClassicChildren(to.path));//emit发送--》//重置清空数组，再赋值数组，相当于刷新最新数据
  }

})
</script>
<style lang='scss' scoped>
.el-menu-horizontal-warp {
  flex: 1;
  overflow: hidden;
  margin-right: 30px;

  :deep(.el-scrollbar__bar.is-vertical) {
    display: none;
  }

  :deep(a) {
    width: 100%;
  }

  .el-menu.el-menu--horizontal {
    display: flex;
    width: 100%;
    height: 100%;
    box-sizing: border-box;
  }
}
</style>